@@ -19,7 +19,7 @@
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  Copyright (C) 2002-2004 Jan-ke Larsson
+  Copyright (C) 2002-2005 Jan-ke Larsson
 
 ************************************************************************/
 
@@ -43,11 +43,15 @@
 #ifdef HAVE_INTTYPES_H
 #  include <inttypes.h>
 #else /* HAVE_INTTYPES_H */
+# ifndef __sgi
+/* IRIX has the following types typedef'd in sys/types.h already,
+ * i.e., _old_ IRIX systems; newer ones have a working inttypes.h */
 typedef signed char               int8_t;
-typedef unsigned char            uint8_t;
 typedef short                    int16_t;
-typedef unsigned short          uint16_t;
 typedef int                      int32_t;
+#endif /* ! __sgi */
+typedef unsigned char            uint8_t;
+typedef unsigned short          uint16_t;
 typedef unsigned int            uint32_t;
 typedef long long                int64_t;
 typedef unsigned long long      uint64_t;
@@ -60,25 +64,12 @@ typedef unsigned long long      uint64_t;
 #define INT32_MAX   (2147483647)
 #endif
 
-#ifdef HAVE_STDBOOL_H
-#include <stdbool.h>
-#endif
-
 #include <gd.h>
 
-#ifdef HAVE_LIBKPATHSEA
+#ifdef HAVE_KPATHSEA_KPATHSEA_H
 # include <kpathsea/kpathsea.h>
-/* boolean is an enum type from kpathsea/types.h loaded in
-   kpathsea/kpathsea.h, use it as fallback */
-# ifndef __bool_true_false_are_defined
-#  define bool boolean
-# endif
 #else
-/* This is really insufficient, currently no path searches will work
-   without kpathsea anyway */
-typedef int bool;
-# define true (bool) 1
-# define false (bool) 0
+# error: kpathsea/kpathsea.h is missing from your system
 #endif
 
 #ifdef HAVE_FT2
@@ -92,6 +83,20 @@ typedef int bool;
 #include <t1lib.h>
 #endif
 
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# ifdef HAVE_KPATHSEA_KPATHSEA_H
+/* boolean is an enum type from kpathsea/types.h loaded in
+   kpathsea/kpathsea.h, use it as fallback */
+#  define bool boolean
+# else
+typedef int bool;
+#  define true (bool) 1
+#  define false (bool) 0
+# endif
+#endif
+
 #ifndef HAVE_VPRINTF
 # ifdef HAVE_DOPRNT
 #  define   vfprintf(stream, message, args)  _doprnt(message, args, stream)
@@ -357,6 +362,7 @@ struct page_list*   NextPPage(void* /* dvi */, struct page_list*);
 #include "commands.h"
 
 void      CreateImage(pixels width, pixels height);
+void      DestroyImage(void);
 void      DrawCommand(unsigned char*, void* /* dvi/vf */); 
 void      DrawPages(void);
 void      WriteImage(char*, int);
@@ -382,7 +388,7 @@ void resetcolorstack(char *);
 void StoreColorStack(struct page_list *tpagep);
 void ReadColorStack(struct page_list *tpagep);
 void StoreBackgroundColor(struct page_list *tpagep);
-void ClearDvipsNam(void);
+void ClearColorNames(void);
 
 
 /**********************************************************************/
@@ -424,6 +430,7 @@ EXTERN struct internal_state {
 #define NO_GHOSTSCRIPT               (1<<17)
 #define NO_GSSAFER                   (1<<18)
 #define BG_TRANSPARENT               (1<<19)
+#define BG_TRANSPARENT_ALPHA         (1<<20)
 EXTERN uint32_t flags INIT(BE_NONQUIET | USE_FREETYPE | USE_LIBT1);
 
 #ifdef DEBUG
