@@ -19,7 +19,7 @@
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  Copyright (C) 2002-2004 Jan-ke Larsson
+  Copyright (C) 2002-2005 Jan-ke Larsson
 
 ************************************************************************/
 
@@ -46,14 +46,14 @@ void LoadFT(int32_t c, struct char_entry * ptr)
 		     glyph_i,              /* glyph index           */
 		     FT_LOAD_RENDER | FT_LOAD_NO_HINTING ))
                                            /* load flags            */
-    Fatal("can't load char_entry %d",c);
+    Fatal("cannot load FT char %d",c);
   ptr->xOffset = -currentfont->face->glyph->bitmap_left*shrinkfactor;
   ptr->yOffset = (currentfont->face->glyph->bitmap_top-1)*shrinkfactor;
   bitmap=currentfont->face->glyph->bitmap;
   DEBUG_PRINT(DEBUG_FT,(" (%dx%d)",bitmap.width,bitmap.rows));
     
   if ((ptr->data = calloc(bitmap.width*bitmap.rows,sizeof(char))) == NULL)
-    Fatal("Unable to allocate image space for char <%c>\n", (char)c);
+    Fatal("unable to allocate image space for char %c", (char)c);
   ptr->w = bitmap.width;
   ptr->h = bitmap.rows;
 
@@ -84,11 +84,15 @@ bool InitFT(struct font_entry * tfontp)
     } 
 # ifdef DEBUG
     else {
+#  ifdef HAVE_FT_LIBRARY_VERSION
       FT_Int      amajor, aminor, apatch;
-      
+
       FT_Library_Version( libfreetype, &amajor, &aminor, &apatch );
-      DEBUG_PRINT(DEBUG_FT,("\n  FREETYPE VERSION: FreeType %d.%d.%d", 
+      DEBUG_PRINT(DEBUG_FT,("\n  LIBFT VERSION: %d.%d.%d", 
 			    amajor, aminor, apatch));
+#  endif
+      DEBUG_PRINT(DEBUG_FT,("\n  FREETYPE VERSION: %d.%d.%d",
+			    FREETYPE_MAJOR, FREETYPE_MINOR, FREETYPE_PATCH));
     }
 # endif
   }
@@ -119,8 +123,9 @@ bool InitFT(struct font_entry * tfontp)
   } 
   if (FT_Set_Char_Size( tfontp->face, /* handle to face object           */
 			0,            /* char_width in 1/64th of points  */
-			(tfontp->d*64)/65536,
-			/* char_height in 1/64th of points */
+			((int64_t)tfontp->d*64*7200)/7227/65536,
+			/* char_height in 1/64th of _big_points, 
+			   not TeX points */
 			tfontp->dpi/shrinkfactor,   /* horizontal resolution */
 			tfontp->dpi/shrinkfactor )) /* vertical resolution   */ {
     Warning("unable to set font size for %s", tfontp->name);
